# Copyright (c) 2005 by Intevation GmbH
# Authors:
# Jonathan Coles <jonathan@intevation.de> 
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with Thuban for details.

"""Raster Layer Properties dialog"""

__version__ = "$Revision$"
# $Source$
# $Id$

from Thuban import _
from Thuban.UI.layerproperties import LayerProperties

from wxPython.wx import *

from string import atoi
from Thuban.Model.color import Color, hex2Color, Color2hex
from Thuban.UI.colordialog import ColorDialog

from Thuban.Model.resource import has_gdal_support, gdal_support_status

ID_CB_MASK = 4002
ID_SELECT_MASK_COLOR = 4003
ID_MASK_COLOR_TEXT = 4004
ID_SPINCTRL_PALINDEX = 4005

class RasterLayerProperties(LayerProperties):

    def __init__(self, parent, name, layer, *args, **kw):
        LayerProperties.__init__(self, parent, name, layer)

        LayerProperties.dialog_layout(self)


    def dialog_layout(self, panel, panelBox):

        info = self.layer.ImageInfo()

        if info is None:
            panelBox.Add(
                wxStaticText(panel, -1, 
                _("GDAL image information unavailable. See About box for details.")),
                0, wxALIGN_LEFT | wxALL, 4)
            return

        
        # Bounding Box
        bbox = self.layer.LatLongBoundingBox()
        if bbox is None:
            text = _("Extent (lat-lon): None")
        else:
            text = _("Extent (lat-lon): (%g, %g, %g, %g)") % tuple(bbox)

        panelBox.Add(wxStaticText(panel, -1, text), 0, wxALIGN_LEFT|wxALL, 4)

        rasterBox = wxStaticBoxSizer(wxStaticBox(panel, -1, 
                    _("Image Properties")), wxVERTICAL)


        rasterBox.Add(
            wxStaticText(panel, -1, 
            _("Source: %s") % self.layer.GetImageFilename()),
            0, wxALIGN_LEFT | wxALL, 4)

        infoBox = wxBoxSizer(wxHORIZONTAL)

        nBands = info["nBands"]

        self.usePalIndex = nBands == 1
        
        infoBox.Add(
            wxStaticText(panel, -1, _("Driver: %s") % info["Driver"]),
            0, wxALIGN_LEFT | wxRIGHT, 10)
        infoBox.Add(
            wxStaticText(panel, -1, _("Size: %ix%i") % info["Size"]),
            0, wxALIGN_LEFT | wxRIGHT, 10)
        infoBox.Add(
            wxStaticText(panel, -1, _("Number of Bands: %i") % nBands),
            0, wxALIGN_LEFT | wxRIGHT, 0)

        rasterBox.Add(infoBox, 0, wxALIGN_LEFT|wxALL, 4)

        # Mask 

        if self.usePalIndex:
            palBox = wxBoxSizer(wxHORIZONTAL)

            self.maskCB = wxCheckBox(panel, ID_CB_MASK, _("Use Mask Index:"))
            palBox.Add(self.maskCB, 0, wxRIGHT, 10)

            spinMin = max(0,   int(info["BandData"][0][0]))
            spinMax = max(255, int(info["BandData"][0][1]) + 1)
            self.palIndex = wxSpinCtrl(self, ID_SPINCTRL_PALINDEX,
                                       min=spinMin, max=spinMax, 
                                       value="0", initial=0)
            self.OnRevert(None)

            palBox.Add(self.palIndex, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT, 10)

            rasterBox.Add(palBox, 0, wxALL, 4)
            panelBox.Add(rasterBox, 1, wxGROW | wxALL, 4)

        elif nBands > 1:
            maskBox = wxBoxSizer(wxHORIZONTAL)

            self.maskCB = wxCheckBox(panel, ID_CB_MASK, _("Use Mask Color:"))
            maskBox.Add(self.maskCB, 0, wxRIGHT, 10)

            self.maskColorText = wxTextCtrl(panel, ID_MASK_COLOR_TEXT, "")
            self.maskColorText.SetMaxLength(7) # '#rrggbb'
            self.OnRevert(None)

            maskBox.Add(self.maskColorText, 0, wxRIGHT, 10)

            self.selectMaskColorButton \
                = wxButton(panel, ID_SELECT_MASK_COLOR, _("Select Mask Color"))

            maskBox.Add(self.selectMaskColorButton, 0, wxALL, 0)

            rasterBox.Add(maskBox, 0, wxALL, 4)
            panelBox.Add(rasterBox, 1, wxGROW | wxALL, 4)

            EVT_BUTTON(self, ID_SELECT_MASK_COLOR, self.OnSelectMaskColor)
            EVT_TEXT(self, ID_MASK_COLOR_TEXT, self.OnMaskColorTextChanged)

        EVT_CHECKBOX(self, ID_CB_MASK, self.OnToggleMask)

        self.maskCB.SetValue(self.layer.UseMask())
        self.OnToggleMask(None)

    def OnTry(self, event):
        if self.usePalIndex:
            self.set_layer_mask_index()
        else:
            self.set_layer_mask_color()

    def OnOK(self, event):
        if self.usePalIndex:
            if self.set_layer_mask_index(): self.Close()
        else:
            if self.set_layer_mask_color(): self.Close()

    def OnRevert(self, event):

        if self.usePalIndex:
            if self.layer.HasMask():
                self.palIndex.SetValue(int(self.layer.MaskColor().red * 255))
            else:
                self.palIndex.SetValue(0)
        else:
            if self.layer.HasMask():
                self.maskColorText.SetValue(Color2hex(self.layer.MaskColor()))
            else:
                self.maskColorText.SetValue("#RRGGBB")

    def OnToggleMask(self, event):
        if self.usePalIndex:
            self.palIndex.Enable(self.maskCB.IsChecked())
        else:
            self.maskColorText.Enable(self.maskCB.IsChecked())
            self.selectMaskColorButton.Enable(self.maskCB.IsChecked())
            self.OnMaskColorTextChanged(None)

    def OnMaskColorTextChanged(self, event):
        if self.get_valid_mask_color() is None:
            self.maskColorText.SetForegroundColour(wxRED)
        else:
            self.maskColorText.SetForegroundColour(wxBLACK)

    def OnSelectMaskColor(self, event):
        dialog = ColorDialog(self)
        if self.maskCB.IsChecked():
            color = self.get_valid_mask_color()
            if color is not None:
                dialog.SetColor(color)

            if dialog.ShowModal() == wxID_OK:
                self.maskColorText.SetValue(Color2hex(dialog.GetColor()))

            dialog.Destroy()

    def get_valid_mask_color(self):
        return hex2Color(self.maskColorText.GetValue())

    def set_layer_mask_color(self):
        if not self.maskCB.IsChecked():
            self.layer.SetUseMask(False)
        else:
            color = self.get_valid_mask_color()
            if color is not None:
                self.layer.SetMaskColor(color, True)
            else:
                wxMessageDialog(self, 
                                "Mask color is not a valid color. The text "
                                "must be in the format '#RRGGBB'.",
                                style = wxOK | wxICON_EXCLAMATION).ShowModal()
                return False

        return True

    def set_layer_mask_index(self):
        if not self.maskCB.IsChecked():
            self.layer.SetUseMask(False)
        else:
            index = self.palIndex.GetValue()
            color = Color(index/255.0, 0, 0)
            self.layer.SetMaskColor(color, True)

        return True

